import { Application } from 'egg'
import { DEFAULT_RULE_CODE } from '../settings'

export default function(app: Application) {
  const { STRING, INTEGER, DATE } = app.Sequelize

  const User = app.model.define('users', {
    id: {
      type: INTEGER,
      primaryKey: true,
      autoIncrement: true
    },
    login_name: {
      type: STRING(60),
      allowNull: false
    },
    password: {
      type: STRING,
      allowNull: false
    },
    display_name: {
      type: STRING(60),
      allowNull: false
    },
    email: {
      type: STRING(100),
      defaultValue: '',
      allowNull: false
    },
    role: {
      type: STRING(50),
      defaultValue: DEFAULT_RULE_CODE,
      allowNull: false
    },
    registered_time: {
      type: DATE,
      defaultValue: new Date()
    },
    last_login_time: {
      type: DATE,
      defaultValue: new Date()
    },
    status: {
      type: INTEGER,
      defaultValue: 1
    },
    is_system_admin: {
      type: INTEGER,
      defaultValue: 0
    },
    avatar: {
      type: STRING,
      allowNull: false,
      defaultValue: ''
    },
    agent: {
      type: STRING,
      allowNull: false,
      defaultValue: ''
    },
    is_deleted: {
      type: INTEGER,
      allowNull: false,
      defaultValue: 0
    }
  })

  return class extends User {
    static readonly tableName = 'nkm_users'

    static associate() {
      app.model.User.belongsTo(app.model.Role, {
        foreignKey: 'role',
        targetKey: 'code',
        as: 'r'
      })
    }
  }
}
